from collections.abc import Iterator
from typing import ClassVar

from .types import AttributeSpec


class OtelGenAISpecification:
    SPEC_VERSION: ClassVar[str] = "1.38.0"
    SUPPORTED_ATTRIBUTES: ClassVar[dict[str, AttributeSpec]] = {}

    def __iter__(self) -> Iterator[AttributeSpec]:
        return iter(self.SUPPORTED_ATTRIBUTES.values())
